#!/usr/bin/env python

'''
Squid Web proxy cache log parsing class.

Contents:
- Parser: squid logfile parser class
  log = Parser(log filehandle)
  - methods:
	log.getlogent()
  - attributes:
    log.utime
    log.elapsed
	log.client
	log.code
	log.status
	log.bytes
	log.method
	log.url
	log.ident
	log.peerstatus
	log.peerhost
	log.mimetype
  - read-only variables:
	error_lines = log.num_error
	lines_processed = log.num_processed
- test: test function

'''

# (c) 1998 Copyright Mark Nottingham
# <mnot@pobox.com>
#
# This software may be freely distributed, modified and used, 
# provided that this copyright notice remain intact.
#
# This software is provided 'as is' without warranty of any kind.


# Squid Logfile Format
# ---------------------
#
# Version 1.1 Access log
# 
# time elapsed host code/status bytes method URL rfc931 peerstatus/peerhost mimetype
#
# rfc931: identd info, - otherwise


__version__ = '0.92'


from string import atoi, atof, split
import sys


class Parser:

	''' Splitting Common Web Logfile Parser '''

	def __init__(self, file_descriptor):
		self.num_processed = 0
		self.num_error = 0
		self._fd = file_descriptor			

	
	def getlogent(self):

		''' Increament location in the log and populate object attributes '''

		line = self._fd.readline()		
		if not line:
			return 0
		self.num_processed = self.num_processed + 1
			
		n = split(line, None)
		if len(n) != 10:
			self.num_error = self.num_error + 1
			self.getlogent()
		else:
			self.utime = int(atof(n[0]))
			self.elapsed = int(atoi(n[1]))
			self.client = n[2]
			(self.code, self.status) = split(n[3], '/', 2) 
			self.bytes = atoi(n[4]) 
			self.method = n[5] 
			self.url = n[6]
			self.ident = n[7]
			(self.peerstatus, self.peerhost) = split(n[8], '/', 2)
			self.mimetype = n[9]
		return 1





			
def test():
	''' basic test suite- modify at will to test all functionality '''
	
	file = sys.stdin
	log = Parser(file)	
	while log.getlogent():
		print "%s %s %s %s" % (log.client, log.code, log.bytes, log.peerhost)
	print "lines: %s" % (log.num_processed)
	print "error: %s" % (log.num_error)
		
		
if __name__ == '__main__':
	test()

