#!/usr/bin/env python

'''
Web Log Referer Typing class.

Contents:
- ReferTyper: logfile referer class
  ref_log = ReferTyper(log object)
  methods:
	- ref_log.getlogent()
  variables:
	- ref_log.siteurl [list of local site urls]
  attributes:
	- res_log.referer_type (MANUAL|LOCAL|OFFSITE|FILE)

- test: test function

'''

# (c) 1998 Copyright Mark Nottingham
# <mnot@pobox.com>
#
# This software may be freely distributed, modified and used,
# provided that this copyright notice remain intact.
#
# This software is provided 'as is' without warranty of any kind.


# Web Log Referer Typing
#
# ReferTyper will determine the type of referer associated with a hit, 
# based on the site URLS that you give it. The possible referer types are:
# 
# - LOCAL -  on one of the sites given
# - MANUAL - a '-' hit
# - FILE - an url beginning with 'file://'
# - OFFSITE - not on one of the sites given, a file or manual hit
#
# the referer attribute of LOCAL hits will be truncated to exclude the 
# scheme and host, and the corresponding attributes will be erased, if
# present.
# 
# This class MUST be fed the output of both a Web logfile parsing module
# that produces a referer attribute, and the WebLogUrlParse class, 
# which populates the individual components that it needs to operate. 
# See the test() for an example.


__version__ = '0.9'


from urlparse import urlparse, urlunparse
from string import lower
import socket


class ReferTyper:
	def __init__(self, log):
		self.log = log
		self._siteurls = {}
		self.siteurl = []


	def __getattr__(self, attr):
		try:
			return getattr(self.log, attr)
		except AttributeError:
			raise AttributeError, attr


	def __setattr__(self, attr, value):
		if attr == 'siteurl':
			for item in value:
				self._siteurls[item] = 1
				try:
					self._siteurls[socket.gethostbyname(item)] = 1
				except socket.error:
					pass
		self.__dict__[attr] = value


	def getlogent(self):

		''' Increment position in the log and populate requested attributes '''

		if self.log.getlogent():

			try:
				delattr(self, 'referer')
				delattr(self, 'ref_scheme')
				delattr(self, 'ref_host')
			except AttributeError:
				pass

			if self.referer == '-':
				self.__dict__['referer_type'] = 'MANUAL'
			elif self._siteurls.has_key(self.ref_host):
				self.__dict__['referer_type'] = 'LOCAL'
				self.__dict__['ref_scheme'] = ''
				self.__dict__['ref_host'] = ''
				self.__dict__['referer'] = urlunparse((	'', 
														'', 
														self.ref_path, 
														self.ref_parameters, 
														self.ref_query, 
														self.ref_fragment))
			elif self.ref_scheme == 'file':
				self.__dict__['referer_type'] = 'FILE'
			else:
				self.__dict__['referer_type'] = 'OFFSITE'

			return 1
		else:
			return 0






def test():

	''' basic test suite- modify at will to test full functionality '''

	import sys, CombinedWebLog, WebLogUrlParse

	file = sys.stdin
	log = CombinedWebLog.Parser(file)

	p_log = WebLogUrlParse.UrlParser(log)
	
	ref_log = ReferTyper(p_log)
	ref_log.siteurl = ['www.foo.com']

	while ref_log.getlogent():
		print "%s	%s	%s" % (ref_log.referer, ref_log.ref_path, ref_log.referer_type)

if __name__ == '__main__':
	test()
