#!/usr/bin/env python

'''
Web Log Url Parsing class.

Contents:
- UrlParser: logfile cleaning class
  clean_log = UrlParser(log object)
  methods:
	- p_log.getlogent()
  variables:
	- p_log.cache_size  [ maximum size of url cache ]
  attributes:
	- (see below)

- test: test function

'''

# (c) 1998 Copyright Mark Nottingham
# <mnot@pobox.com>
#
# This software may be freely distributed, modified and used,
# provided that this copyright notice remain intact.
#
# This software is provided 'as is' without warranty of any kind.


#   url_scheme        /       ref_scheme
#   url_host          /       ref_host
#   url_path          /       ref_path
#   url_parameters    /       ref_parameters
#   url_query         /       ref_query
#   url_fragment      /       ref_fragment


__version__ = '0.91'



from urlparse import urlparse, urlunparse


class UrlParser:
	def __init__(self, log):
		self.log = log
		self.set_url = 0
		self.set_ref = 0
		self.cache_size = 5000
		self._cache = {'url': {}, 'ref': {}}


	def __getattr__(self, attr):
		try:
			return getattr(self.log, attr)
		except AttributeError:
			raise AttributeError, attr


	def getlogent(self):

		''' Increment position in the log and populate requested attributes '''

		if self.log.getlogent():
			self.__dict__['url'] = self._parse(self.log.url, 'url')
			try:
				self.__dict__['referer'] = self._parse(self.log.referer, 'ref')
			except AttributeError:
				pass
			return 1
		else:
			return 0


	def _parse(self, url, url_type):

		if len(self._cache[url_type]) > self.cache_size:
			self._cache[url_type] = {}
			
		try:
			_result = self._cache[url_type][url]
		except KeyError:
			(	_scheme,
			 	_host,
			 	_path,
			 	_param,
			 	_query,
			 	_frag) = urlparse(url)
				
			_result = (	_scheme, 
						_host, 
						_path, 
						_param,
						_query,
						_frag	)
			self._cache[url_type][url] = _result

		self.__dict__[url_type + "_scheme"] = _result[0]
		self.__dict__[url_type + "_host"] = _result[1]
		self.__dict__[url_type + "_path"] = _result[2]
		self.__dict__[url_type + "_parameters"] = _result[3]
		self.__dict__[url_type + "_query"] = _result[4]
		self.__dict__[url_type + "_fragment"] = _result[5]

		return(urlunparse(_result))



def test():

	''' basic test suite- modify at will to test full functionality '''

	import sys, CombinedWebLog

	file = sys.stdin
	log = CombinedWebLog.Parser(file)

	p_log = UrlParser(log)

	while p_log.getlogent():
		print "%s\n%s %s %s" % (log.url, p_log.url, p_log.url_path, p_log.url_query)
		print "%s\n%s %s %s" % (log.referer, p_log.referer, p_log.ref_host, p_log.ref_path)
		print



if __name__ == '__main__':
	test()
